package cn.iocoder.yudao.module.depository.service.warehouse.impl;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseAreaMapper;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaService;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionBackVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 库区库域 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class WarehouseAreaServiceImpl extends AbstractService<WarehouseAreaMapper, WarehouseAreaDO> implements WarehouseAreaService  {

    @Resource
    private WarehouseAreaMapper warehouseAreaMapper;

    @Resource
    private WarehouseAreaPositionService warehouseAreaPositionService;

    @Override
    public Long createWarehouseArea(WarehouseAreaCreateReqVO createReqVO) {
        // 插入
        WarehouseAreaDO warehouseArea = WarehouseAreaConvert.INSTANCE.convert(createReqVO);
        warehouseAreaMapper.insert(warehouseArea);
        // 返回
        return warehouseArea.getId();
    }

    @Override
    public void updateWarehouseArea(WarehouseAreaUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateWarehouseAreaExists(updateReqVO.getId());
        // 更新
        WarehouseAreaDO updateObj = WarehouseAreaConvert.INSTANCE.convert(updateReqVO);
        warehouseAreaMapper.updateById(updateObj);
    }

    @Override
    public void deleteWarehouseArea(Long id) {
        // 校验存在
        this.validateWarehouseAreaExists(id);
        // 删除
        warehouseAreaMapper.deleteById(id);
    }

    private void validateWarehouseAreaExists(Long id) {
        if (warehouseAreaMapper.selectById(id) == null) {
            throw exception(WAREHOUSE_AREA_NOT_EXISTS);
        }
    }

    @Override
    public WarehouseAreaDO getWarehouseArea(Long id) {
        return warehouseAreaMapper.selectById(id);
    }

    @Override
    public List<WarehouseAreaDO> getWarehouseAreaList(Collection<Long> ids) {
        return warehouseAreaMapper.selectBatchIds(ids);
    }

    @Override
    public List<WarehouseAreaDO> getWarehouseAreaByParentIds(Collection<Long> ids) {
        return warehouseAreaMapper.selectList(new LambdaQueryWrapperX<WarehouseAreaDO>()
            .in(WarehouseAreaDO::getPid, ids)
        );
    }

    @Override
    public PageResult<WarehouseAreaDO> getWarehouseAreaPage(WarehouseAreaQueryVO query, PageVO page) {
        return warehouseAreaMapper.selectPage(page, query);
    }

    @Override
    public List<WarehouseAreaDO> getWarehouseAreaList(WarehouseAreaQueryVO query) {
        return warehouseAreaMapper.selectListV1(query);
    }

    @Override
    public List<WarehouseAreaBackVO> getWarehouseAreaListV1(WarehouseAreaQueryVO query) {
        List<WarehouseAreaDO> list = getWarehouseAreaList(query);
        List<WarehouseAreaBackVO> dataList = WarehouseAreaConvert.INSTANCE.convertList(list);
        matchChild(dataList, true);
        return dataList;
    }

    @Override
    public PageResult<WarehouseAreaBackVO> getWarehouseAreaPageV1(WarehouseAreaQueryVO query, PageVO page) {
        PageResult<WarehouseAreaDO> pageResult = warehouseAreaMapper.selectPage(page, query);
        PageResult<WarehouseAreaBackVO> backPageResult = WarehouseAreaConvert.INSTANCE.convertPage(pageResult);
        List<WarehouseAreaBackVO> dataList = backPageResult.getList();
        matchChild(dataList, false);
        return backPageResult;
    }

    /**
     * 匹配子类
     * @param dataList  库域列表
     * @param containsPosition  是否包含仓位信息
     */
    private void matchChild(List<WarehouseAreaBackVO> dataList, boolean containsPosition) {
        if(CollectionUtil.isNotEmpty(dataList)) {
            List<Long> idList = dataList.stream()
                    .map(WarehouseAreaBackVO::getId)
                    .collect(Collectors.toList());

            Map<Long, List<WarehouseAreaPositionBackVO>> areaPositionMap = null;
            if(containsPosition) {
                //查询库域下的所有库位
                List<WarehouseAreaPositionDO> positionDOList = warehouseAreaPositionService.getByDomainIdList(idList);
                List<WarehouseAreaPositionBackVO> positionBackList = WarehouseAreaPositionConvert.INSTANCE.convertList(positionDOList);

                //找出所有父库位
                List<WarehouseAreaPositionBackVO> parentList = positionBackList.stream()
                        .filter(t -> t.getPid() == 0)
                        .collect(Collectors.toList());
                //positionBackList是子库位
                positionBackList.removeAll(parentList);
                //子库位以父id分组
                Map<Long, List<WarehouseAreaPositionBackVO>> childMap = positionBackList.stream()
                        .collect(Collectors.groupingBy(WarehouseAreaPositionBackVO::getPid));
                for (WarehouseAreaPositionBackVO parentDo : parentList) {
                    List<WarehouseAreaPositionBackVO> childs = childMap.get(parentDo.getId());
                    parentDo.setChildren(childs);
                }

                //父库位以库区id分组
                areaPositionMap = parentList.stream()
                        .collect(Collectors.groupingBy(WarehouseAreaPositionBackVO::getAreaId));

            }

            List<WarehouseAreaDO> list = getWarehouseAreaByParentIds(idList);
            Map<Long, List<WarehouseAreaDO>> childGroupMap = list.stream()
                    .collect(Collectors.groupingBy(WarehouseAreaDO::getPid));
            for (WarehouseAreaBackVO backVO : dataList) {
                List<WarehouseAreaDO> childs = childGroupMap.get(backVO.getId());
                List<WarehouseAreaBackVO> childList =  WarehouseAreaConvert.INSTANCE.convertList(childs);
                backVO.setChildren(childList);

                if(containsPosition && CollectionUtil.isNotEmpty(childList)) {
                    for (WarehouseAreaBackVO  areaVo : childList) {
                        List<WarehouseAreaPositionBackVO> areaPositionList = areaPositionMap.get(areaVo.getId());
                        areaVo.setPositionList(areaPositionList);
                    }
                }
            }
        }
    }
}
