package cn.iocoder.yudao.module.depository.controller.admin.warehouse;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaUpdateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionBackVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaConvert;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 库区库域")
@RequestMapping("/ecw/warehouse-area")
public class WarehouseAreaController {

    @Resource
    private WarehouseAreaService warehouseAreaService;


    @PostMapping("/create")
    @ApiOperation("创建库区库域")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:create')")
    public CommonResult<Long> createWarehouseArea(@Valid @RequestBody WarehouseAreaCreateReqVO createReqVO) {
        return success(warehouseAreaService.createWarehouseArea(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新库区库域")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:update')")
    public CommonResult<Boolean> updateWarehouseArea(@Valid @RequestBody WarehouseAreaUpdateReqVO updateReqVO) {
        warehouseAreaService.updateWarehouseArea(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除库区库域")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:delete')")
    public CommonResult<Boolean> deleteWarehouseArea(@RequestParam("id") Long id) {
        warehouseAreaService.deleteWarehouseArea(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得库区库域")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:query')")
    public CommonResult<WarehouseAreaBackVO> getWarehouseArea(@RequestParam("id") Long id) {
        WarehouseAreaDO warehouseArea = warehouseAreaService.getWarehouseArea(id);
        return success(WarehouseAreaConvert.INSTANCE.convert(warehouseArea));
    }

    @GetMapping("/list")
    @ApiOperation("获得库区库域列表")
//    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:query')")
    public CommonResult<List<WarehouseAreaBackVO>> getWarehouseAreaList(WarehouseAreaQueryVO query) {
        List<WarehouseAreaBackVO> list = warehouseAreaService.getWarehouseAreaListV1(query);;
        return success(list);
    }

    @GetMapping("/page")
    @ApiOperation("获得库区库域分页")
//    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:query')")
    public CommonResult<PageResult<WarehouseAreaBackVO>> getWarehouseAreaPage(@Valid WarehouseAreaQueryVO query, PageVO page) {
        PageResult<WarehouseAreaBackVO> pageResult = warehouseAreaService.getWarehouseAreaPageV1(query, page);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出库区库域 Excel")
//    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:export')")
    @OperateLog(type = EXPORT)
    public void exportWarehouseAreaExcel(@Valid WarehouseAreaQueryVO query,
              HttpServletResponse response) throws IOException {
        List<WarehouseAreaDO> list = warehouseAreaService.getWarehouseAreaList(query);
        // 导出 Excel
        List<WarehouseAreaBackVO> datas = WarehouseAreaConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "库区库域.xls", "数据", WarehouseAreaBackVO.class, datas);
    }

    @GetMapping("/getByWarehouseId")
    @ApiOperation("获得仓库查询库域")
//    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area:query')")
    public CommonResult<List<WarehouseAreaBackVO>> getWarehouseAreaPage(WarehouseAreaQueryVO query) {
        List<WarehouseAreaBackVO> list = warehouseAreaService.getWarehouseAreaListV1(query);
        return success(list);
    }



}
