package cn.iocoder.yudao.module.depository.controller.app.warehouse;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaService;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaUpdateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionBackVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaConvert;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "app-web - 库区库域")
@RequestMapping("/ecw/warehouse-area")
public class AppWarehouseAreaController {

    @Resource
    private WarehouseAreaService warehouseAreaService;
    @Resource
    private WarehouseAreaPositionService warehouseAreaPositionService;


    @GetMapping("/get")
    @ApiOperation("获得库区库域")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<WarehouseAreaBackVO> getWarehouseArea(@RequestParam("id") Long id) {
        WarehouseAreaDO warehouseArea = warehouseAreaService.getWarehouseArea(id);
        return success(WarehouseAreaConvert.INSTANCE.convert(warehouseArea));
    }

    @GetMapping("/list")
    @ApiOperation("获得库区库域列表")
    @PreAuthenticated
    public CommonResult<List<WarehouseAreaBackVO>> getWarehouseAreaList(WarehouseAreaQueryVO query) {
        List<WarehouseAreaDO> list = warehouseAreaService.getWarehouseAreaList(query);
        List<WarehouseAreaBackVO> dataList = WarehouseAreaConvert.INSTANCE.convertList(list);
        matchChild(dataList, false);
        return success(dataList);
    }

    @GetMapping("/page")
    @ApiOperation("获得库区库域分页")
    @PreAuthenticated
    public CommonResult<PageResult<WarehouseAreaBackVO>> getWarehouseAreaPage(@Valid WarehouseAreaQueryVO query, PageVO page) {
        PageResult<WarehouseAreaDO> pageResult = warehouseAreaService.getWarehouseAreaPage(query, page);
        PageResult<WarehouseAreaBackVO> backPageResult = WarehouseAreaConvert.INSTANCE.convertPage(pageResult);
        List<WarehouseAreaBackVO> dataList = backPageResult.getList();
        matchChild(dataList, false);
        return success(backPageResult);
    }

    @GetMapping("/getByWarehouseId")
    @ApiOperation("获得仓库查询库域")
    @PreAuthenticated
    public CommonResult<List<WarehouseAreaBackVO>> getWarehouseAreaPage(WarehouseAreaQueryVO query) {
        List<WarehouseAreaDO> list = warehouseAreaService.getWarehouseAreaList(query);
        // 导出 Excel
        List<WarehouseAreaBackVO> datas = WarehouseAreaConvert.INSTANCE.convertList(list);
        matchChild(datas, true);
        return success(datas);
    }

    /**
     * 匹配子类
     * @param dataList  库域列表
     * @param containsPosition  是否包含仓位信息
     */
    private void matchChild(List<WarehouseAreaBackVO> dataList, boolean containsPosition) {
        if(CollectionUtil.isNotEmpty(dataList)) {
            List<Long> idList = dataList.stream()
                    .map(WarehouseAreaBackVO::getId)
                    .collect(Collectors.toList());

            Map<Long, List<WarehouseAreaPositionBackVO>> areaPositionMap = null;
            if(containsPosition) {
                //查询库域下的所有库位
                List<WarehouseAreaPositionDO> positionDOList = warehouseAreaPositionService.getByDomainIdList(idList);
                List<WarehouseAreaPositionBackVO> positionBackList = WarehouseAreaPositionConvert.INSTANCE.convertList(positionDOList);

                //找出所有父库位
                List<WarehouseAreaPositionBackVO> parentList = positionBackList.stream()
                        .filter(t -> t.getPid() == 0)
                        .collect(Collectors.toList());
                //positionBackList是子库位
                positionBackList.removeAll(parentList);
                //子库位以父id分组
                Map<Long, List<WarehouseAreaPositionBackVO>> childMap = positionBackList.stream()
                        .collect(Collectors.groupingBy(WarehouseAreaPositionBackVO::getPid));
                for (WarehouseAreaPositionBackVO parentDo : parentList) {
                    List<WarehouseAreaPositionBackVO> childs = childMap.get(parentDo.getId());
                    parentDo.setChildren(childs);
                }

                //父库位以库区id分组
                areaPositionMap = parentList.stream()
                        .collect(Collectors.groupingBy(WarehouseAreaPositionBackVO::getAreaId));

            }

            List<WarehouseAreaDO> list = warehouseAreaService.getWarehouseAreaByParentIds(idList);
            Map<Long, List<WarehouseAreaDO>> childGroupMap = list.stream()
                    .collect(Collectors.groupingBy(WarehouseAreaDO::getPid));
            for (WarehouseAreaBackVO backVO : dataList) {
                List<WarehouseAreaDO> childs = childGroupMap.get(backVO.getId());
                List<WarehouseAreaBackVO> childList =  WarehouseAreaConvert.INSTANCE.convertList(childs);
                backVO.setChildren(childList);

                if(containsPosition) {
                    for (WarehouseAreaBackVO  areaVo : childList) {
                        List<WarehouseAreaPositionBackVO> areaPositionList = areaPositionMap.get(areaVo.getId());
                        areaVo.setPositionList(areaPositionList);
                    }
                }
            }
        }
    }

}
